@@ -18,6 +18,7 @@ from __future__ import absolute_import, print_function
 import os
 import re
 import tempfile
+import shutil
 import base64
 
 import yaml
@@ -49,14 +50,15 @@ class EasyVaultException(Exception):
     Base exception for all exceptions raised by the
     :class:`~easy_vault.EasyVault` class.
 
-    Derived from :exc:`py:Exception`.
+    Derived from :exc:`~py:Exception`.
     """
     pass
 
 
 class EasyVaultFileError(EasyVaultException):
     """
-    Exception indicating file I/O errors with a vault file.
+    Exception indicating file I/O errors with a vault file or with a temporary
+    file (that is used when writing the vault file).
 
     Derived from :exc:`EasyVaultException`.
     """
@@ -74,7 +76,7 @@ class EasyVaultDecryptError(EasyVaultException):
 
 class EasyVaultEncryptError(EasyVaultException):
     """
-    Exception indicating that an unencrypted vault file could not be encrypted.
+    Exception indicating that a decrypted vault file could not be encrypted.
 
     Derived from :exc:`EasyVaultException`.
     """
@@ -106,8 +108,13 @@ class EasyVault(object):
     well suited for handling huge files. It is really meant for vault files:
     Files that keep secrets, but not huge data.
 
-    The password is converted to a symmetric key which is then used for
-    encryption and decryption of the vault file.
+    The password may be provided or not (`None`).
+    If the password is provided, it is converted to a symmetric key which is
+    then used for encrypting and decrypting the vault file itself and for
+    decrypting the vault file content upon access.
+    If the password is not provided, encryption and decryption of the vault file
+    is rejected and access to the vault file content requires that the vault
+    file is in the decrypted state.
 
     The encryption package used by this class is pluggable. The default
     implementation uses the symmetric key support from the
@@ -122,7 +129,7 @@ class EasyVault(object):
     * :meth:`decrypt_data` - Decrypt encrypted data with a symmetric key.
     """
 
-    def __init__(self, filepath, password):
+    def __init__(self, filepath, password=None):
         """
         Parameters:
 
@@ -130,20 +137,28 @@ class EasyVault(object):
             Path name of the vault file.
 
           password (:term:`unicode string`):
-            Password for encrypting and decrypting the vault file.
+            Password for encrypting and decrypting the vault file, or `None`
+            if not provided.
+
+        Raises:
+          TypeError: Type error in arguments or in return of pluggable
+            encryption function.
         """
         self._filepath = filepath
-        if not isinstance(password, six.string_types):
-            raise TypeError(
-                "The password argument is not a unicode string, but: {t}".
-                format(t=type(password)))
-        key = self.generate_key(password)
-        if not isinstance(key, six.binary_type):
-            raise TypeError(
-                "The return value of generate_key() is not a byte string, "
-                "but: {t}".
-                format(t=type(key)))
-        self._key = key
+        if password is None:
+            self._key = None
+        else:
+            if not isinstance(password, six.string_types):
+                raise TypeError(
+                    "The password argument is not a unicode string, but: {t}".
+                    format(t=type(password)))
+            key = self.generate_key(password)
+            if not isinstance(key, six.binary_type):
+                raise TypeError(
+                    "The return value of generate_key() is not a byte string, "
+                    "but: {t}".
+                    format(t=type(key)))
+            self._key = key
 
     @property
     def filepath(self):
@@ -152,6 +167,13 @@ class EasyVault(object):
         """
         return self._filepath
 
+    @property
+    def password_provided(self):
+        """
+        bool: Indicates whether a vault password was provided.
+        """
+        return self._key is not None
+
     def is_encrypted(self):
         """
         Test whether the vault file is encrypted by easy-vault.
@@ -159,14 +181,15 @@ class EasyVault(object):
         This is done by checking for the unique easy-vault header in the first
         line of the vault file.
 
+        This method does not require a vault password to be provided.
+
         Returns:
           bool: Boolean indicating whether the vault file is encrypted by
           easy-vault.
 
         Raises:
-          EasyVaultFileError: Error with the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file.
         """
-
         try:
             with open(self._filepath, 'rb') as fp:
                 first_bline = fp.readline()  # Including trailing newline
@@ -175,9 +198,10 @@ class EasyVault(object):
                 "Cannot open vault file {fn} for reading: {exc}".
                 format(fn=self._filepath, exc=exc))
             new_exc.__cause__ = None
-            raise new_exc  # VaultFileOpenError
-
-        m = HEADER_PATTERN.match(first_bline)
+            raise new_exc  # EasyVaultFileError
+        # On Windows, match() does not tolerate CRLF line endings
+        first_bline_s = first_bline.strip(b'\n').strip(b'\r')
+        m = HEADER_PATTERN.match(first_bline_s)
         if m is None:
             return False
         if m.group('moniker') != HEADER_MONIKER:
@@ -188,17 +212,27 @@ class EasyVault(object):
         """
         Encrypt the vault file.
 
-        The vault file must be unencrypted (i.e. not encrypted by easy-vault).
+        The vault file must be decrypted (i.e. not encrypted by easy-vault).
+
+        This method requires a vault password to be provided.
 
         Raises:
-          EasyVaultFileError: Error with the vault file or a temporary file
-          EasyVaultEncryptError: Error encrypting the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file or a
+            temporary file.
+          :exc:`EasyVaultEncryptError`: Error encrypting the vault file.
         """
+        if not self.password_provided:
+            raise EasyVaultEncryptError(
+                "Cannot encrypt vault file {fn}: "
+                "No password was provided".
+                format(fn=self._filepath))
 
         try:
-            with open(self._filepath, 'rb', encoding=None) as fp:
+            with open(self._filepath, 'rb') as fp:
                 first_bline = fp.readline()  # Including trailing newline
-                m = HEADER_PATTERN.match(first_bline)
+                first_bline_s = first_bline.strip(b'\n').strip(b'\r')
+                # On Windows, match() does not tolerate CRLF line endings
+                m = HEADER_PATTERN.match(first_bline_s)
                 if m and m.group('moniker') == HEADER_MONIKER:
                     raise EasyVaultEncryptError(
                         "Cannot encrypt vault file {fn}: "
@@ -231,29 +265,40 @@ class EasyVault(object):
 
         The vault file must be encrypted by easy-vault.
 
+        This method requires a vault password to be provided.
+
         Raises:
-          EasyVaultFileError: Error with the vault file or a temporary file
-          EasyVaultDecryptError: Error decrypting the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file or a
+            temporary file.
+          :exc:`EasyVaultDecryptError`: Error decrypting the vault file.
         """
         clear_bdata = self._get_bytes_from_encrypted()
         write_file(self._filepath, clear_bdata)
 
     def _get_bytes_from_encrypted(self):
         """
-        Get the unencrypted data from an encrypted vault file.
+        Get the content of an encrypted vault file by decrypting the content
+        upon access and leaving the file unchanged.
 
         Returns:
-          :term:`byte string`: Unencrypted data from the vault file.
+          :term:`byte string`: Decrypted content of the vault file.
 
         Raises:
-          EasyVaultFileError: Error with the vault file
-          EasyVaultDecryptError: Error decrypting the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file.
+          :exc:`EasyVaultDecryptError`: Error decrypting the vault file.
         """
+        if not self.password_provided:
+            raise EasyVaultDecryptError(
+                "Cannot decrypt vault file {fn}: "
+                "No password was provided".
+                format(fn=self._filepath))
 
         try:
             with open(self._filepath, 'rb') as fp:
                 first_bline = fp.readline()  # Including trailing newline
-                m = HEADER_PATTERN.match(first_bline)
+                first_bline_s = first_bline.strip(b'\n').strip(b'\r')
+                # On Windows, match() does not tolerate CRLF line endings
+                m = HEADER_PATTERN.match(first_bline_s)
                 if m is None or m.group('moniker') != HEADER_MONIKER:
                     raise EasyVaultDecryptError(
                         "Cannot decrypt vault file {fn}: "
@@ -279,13 +324,13 @@ class EasyVault(object):
 
     def _get_bytes_from_clear(self):
         """
-        Get the data from an unencrypted vault file.
+        Get the content of a decrypted vault file.
 
         Returns:
-          :term:`byte string`: Unencrypted data from the vault file.
+          :term:`byte string`: Decrypted content of the vault file.
 
         Raises:
-          EasyVaultFileError: Error with the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file.
         """
         try:
             with open(self._filepath, 'rb') as fp:
@@ -300,17 +345,21 @@ class EasyVault(object):
 
     def get_bytes(self):
         """
-        Get the unencrypted content of the vault file as a Byte sequence.
+        Get the decrypted content of the vault file as a Byte sequence.
 
-        The vault file may be encrypted or unencrypted.
+        The vault file may be in the encrypted or decrypted state and remains
+        unchanged.
+
+        If the vault file is in the encrypted state, the object this method
+        is called on must have been created with a vault password.
 
         Returns:
-          :term:`byte string`: Unencrypted content of the vault file, as a Byte
+          :term:`byte string`: Decrypted content of the vault file, as a Byte
           sequence.
 
         Raises:
-          EasyVaultFileError: Error with the vault file
-          EasyVaultDecryptError: Error decrypting the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file.
+          :exc:`EasyVaultDecryptError`: Error decrypting the vault file.
         """
         if self.is_encrypted():
             return self._get_bytes_from_encrypted()
@@ -318,17 +367,21 @@ class EasyVault(object):
 
     def get_text(self):
         """
-        Get the unencrypted content of the vault file as a Unicode string.
+        Get the decrypted content of the vault file as a Unicode string.
 
-        The vault file may be encrypted or unencrypted.
+        The vault file may be in the encrypted or decrypted state and remains
+        unchanged.
+
+        If the vault file is in the encrypted state, the object this method
+        is called on must have been created with a vault password.
 
         Returns:
-          :term:`unicode string`: Unencrypted content of the vault file, as a
+          :term:`unicode string`: Decrypted content of the vault file, as a
           Unicode string.
 
         Raises:
-          EasyVaultFileError: Error with the vault file
-          EasyVaultDecryptError: Error decrypting the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file.
+          :exc:`EasyVaultDecryptError`: Error decrypting the vault file.
         """
         bdata = self.get_bytes()
         udata = bdata.decode('utf-8')
@@ -336,18 +389,23 @@ class EasyVault(object):
 
     def get_yaml(self):
         """
-        Get the unencrypted content of a YAML-formatted vault file as a YAML
+        Get the decrypted content of a YAML-formatted vault file as a YAML
         object.
 
-        The vault file may be encrypted or unencrypted.
+        The vault file may be in the encrypted or decrypted state and remains
+        unchanged.
+
+        If the vault file is in the encrypted state, the object this method
+        is called on must have been created with a vault password.
 
         Returns:
           dict or list: Top-level object of the YAML-formatted vault file.
 
         Raises:
-          EasyVaultFileError: Error with the vault file or a temporary file
-          EasyVaultYamlError: YAML syntax error in the vault file
-          EasyVaultDecryptError: Error decrypting the vault file
+          :exc:`EasyVaultFileError`: I/O error with the vault file or a
+            temporary file.
+          :exc:`EasyVaultYamlError`: YAML syntax error in the vault file.
+          :exc:`EasyVaultDecryptError`: Error decrypting the vault file.
         """
         clear_bdata = self.get_bytes()
         try:
@@ -437,7 +495,7 @@ class EasyVault(object):
           :term:`byte string`: The clear data.
 
         Raises:
-          EasyVaultDecryptError: Error decrypting the vault file
+          :exc:`EasyVaultDecryptError`: Error decrypting the vault file.
         """
         f = Fernet(key)
         try:
@@ -470,7 +528,7 @@ def write_file(filepath, data):
         The data to be written.
 
     Raises:
-      EasyVaultFileError: Error with the vault file or a temporary file
+      :exc:`EasyVaultFileError`: I/O error with the file.
     """
     assert isinstance(data, six.binary_type), type(data)
     try:
@@ -486,7 +544,10 @@ def write_file(filepath, data):
         raise new_exc  # EasyVaultFileError
 
     try:
-        os.rename(tfpath, filepath)
+        # On Windows, the temp file may be on a different drive than the
+        # original file, so os.rename() cannot be used.
+        shutil.copy(tfpath, filepath)
+        os.remove(tfpath)
     except (OSError, IOError) as exc:
         new_exc = EasyVaultFileError(
             "Cannot rename temporary file {tfn} to vault file {fn}: {exc}".
